package org.netkernel.lang.dpml.ast.impl;

import org.netkernel.lang.dpml.ast.ILocation;
import org.netkernel.lang.dpml.ast.IReferenceVariable;
import org.netkernel.lang.dpml.ast.IScope;
import org.netkernel.lang.dpml.ast.IVariable;
import org.netkernel.lang.dpml.endpoint.RuntimeState;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.INKFResponseReadOnly;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.nkf.impl.NKFResponseReadOnlyImpl;
import org.netkernel.layer0.urii.ValueSpace;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.impl.ExpiryFactory;
import org.netkernel.request.impl.MetaImpl;
import org.netkernel.urii.ISpace;

public class ArgumentReference extends VariableImpl implements IReferenceVariable
{
	private final String mArgumentName;
	private final int mVerb;
	
	public ArgumentReference(String aName, IScope aScope, ILocation aLocation, String aArgumentName, int aVerb)
	{	super(aName, aScope, aLocation);
		mArgumentName=aArgumentName;
		mVerb=aVerb;
	}
	
	public boolean equals(Object aOther)
	{	return aOther instanceof ArgumentReference && ((ArgumentReference)aOther).mArgumentName.equals(mArgumentName);
	}
	
	public String getArgumentName()
	{	return mArgumentName;
	}
	
	public int getVerb()
	{	return mVerb;
	}

	public IVariable resolve(INKFRequestContext aContext, RuntimeState aState) throws NKFException
	{
		RuntimeState rs=aState;
		Object var=null;
		while (rs!=null)
		{	var=rs.getArgument(mArgumentName);
			if (var!=null)
			{	break;
			}
			else
			{	rs=rs.getParent();
			}
		}
		IVariable result;
		switch(mVerb)
		{
			case INKFRequestReadOnly.VERB_EXISTS:
			{	boolean exists;
				if (var instanceof IVariable)
				{	exists=true;
				}
				else if (var instanceof String)
				{	try
					{	INKFRequest req=aContext.createRequest((String)var);
						req.setVerb(INKFRequestReadOnly.VERB_EXISTS);
						req.setRequestScope(rs.getArgumentScope());
						exists=(Boolean)aContext.issueRequest(req);
					}
					catch (ClassCastException e)
					{	exists=true;
					}
					catch (Exception e)
					{	exists=false;
					}
				}
				else
				{	exists=false;
				}
				result=new LiteralValue("arg:"+mArgumentName, getScope(), getLocation(), exists, new MetaImpl(ExpiryFactory.getDependentExpiry()));
				break;
			}
			case INKFRequestReadOnly.VERB_SOURCE:
			{	if (var instanceof IVariable)
				{	result=(IVariable)var;
				}
				else if (var instanceof String)
				{	String identifier=(String)var;
					boolean isPBV=isArgumentByValue(rs.getArgumentScope(), identifier);
					if (isPBV)
					{	INKFRequest req=aContext.createRequest(identifier);
						req.setRequestScope(rs.getArgumentScope());
						INKFResponseReadOnly r=aContext.issueRequestForResponse(req);
						result=new LiteralValue("arg:"+mArgumentName, getScope(), getLocation(), r.getRepresentation(),((NKFResponseReadOnlyImpl)r).getMeta());
					}
					else
					{	result=new LiteralIndentifier("arg:"+mArgumentName, getScope(), getLocation(), identifier, true);
					}
				}
				else
				{	NKFException e=new NKFException("arg:"+mArgumentName+" not found");
					result=new LiteralValue("arg:"+mArgumentName, getScope(), getLocation(), e,new MetaImpl(ExpiryFactory.getDependentExpiry()));
				}
				break;
			}
			default:
			{
				result=new LiteralValue("arg:"+mArgumentName, getScope(), getLocation(), "unsupported verb", new MetaImpl(ExpiryFactory.getDependentExpiry()));
			}
		}
		return result;
	}
		
	private static boolean isArgumentByValue(IRequestScopeLevel aScope, String aIdentifier)
	{
		IRequestScopeLevel scope=aScope;
		ValueSpace vs=null;
		while(scope!=null)
		{	ISpace s=scope.getSpace();
			if (s instanceof ValueSpace)
			{	vs=(ValueSpace)s;
				break;
			}
			else
			{	scope=scope.getParent();
			}
		}
		boolean result=false;
		if (vs!=null)
		{	for (int j=0; j<vs.getSize(); j++)
			{	String id=vs.getIdentifier(j);
				if (id.equals(aIdentifier))
				{	result=true;
					break;
				}
			}
		}
		return result;
	}
}
