package org.netkernel.lang.dpml.endpoint;

import org.netkernel.lang.dpml.ast.*;
import org.netkernel.lang.dpml.ast.impl.*;
import org.netkernel.lang.dpml.representation.DPMLRepresentation;
import org.netkernel.layer0.nkf.*;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import org.netkernel.request.IRequestResponseFields;

public class DPMLRuntime extends StandardAccessorImpl
{
	public DPMLRuntime()
	{	this.declareThreadSafe();
		this.declareUnhandledExceptionsExpired(false);
	}
	
	public void onRequest(INKFRequestContext aContext) throws Exception
	{
		if (aContext.getThisRequest().getVerb()==INKFRequestReadOnly.VERB_META)
		{	super.onMeta(aContext);
		}
		else
		{	// get the script
			String scriptIdentifier = aContext.getThisRequest().getArgumentValue("operator");
			aContext.setCWU(scriptIdentifier);
			INKFResponseReadOnly<DPMLRepresentation> scriptResponse=aContext.sourceForResponse(scriptIdentifier, DPMLRepresentation.class);
			DPMLRepresentation script = scriptResponse.getRepresentation();
			
			//initialise state for this runtime
			RuntimeState state=new RuntimeState(aContext,scriptIdentifier);

			//determine which closure we want to run
			IVariable rootResponse = script.getRootScope();
			if (aContext.getThisRequest().argumentExists("response"))
			{	String responseClosure=aContext.getThisRequest().getArgumentValue("response");
				if (rootResponse instanceof IScope)
				{	rootResponse=((Closure) rootResponse).resolveAssignment(responseClosure, state);
					if (rootResponse==null)
					{	throw aContext.createFormattedException("EX_DPML_RUNTIME","MSG_RUN_NOT_FOUND",null,null,responseClosure);
					}
				}
				else
				{	throw aContext.createFormattedException("EX_DPML_RUNTIME","MSG_ARG_RESPONSE_NOT_CLOSURE",null,null);
				}
			}
			
			//execute
			IVariable var=state.getAssignment(rootResponse.getScope(), rootResponse.getName());
			if (var==null)
			{	var=DPMLUtils.dereference(rootResponse,state,aContext);
			}
			INKFResponse result=null;
			if (var instanceof IIdentifierVariable)
			{	IValue value=DPMLUtils.requestIdentifier((IIdentifierVariable)var, aContext, state, aContext.getThisRequest().getRepresentationClass());
				result=DPMLUtils.createResponseWithMetaData(value,aContext);
			}
			else if (var instanceof IValueVariable)
			{	IValueVariable valueVar = (IValueVariable)var;
				INKFRequestReadOnly thisReq=aContext.getThisRequest();
				IValue value=valueVar.getValue(aContext,state,thisReq.getRepresentationClass());
				if (value!=null)
				{	result=DPMLUtils.createResponseWithMetaData(value,aContext);
					IRequestResponseFields responseHeaders=value.getMeta().getUserMetaData();
					if (scriptResponse!=null && responseHeaders.getValue(ResponseAccessor.HEADER_EXPIRY)!=null)
					{	DPMLUtils.setResponseExpiry(result,responseHeaders,scriptResponse);
					}
				}
			}
		}
	}
}
