package org.netkernel.lang.dpml.endpoint;

import org.netkernel.lang.dpml.ast.IValue;
import org.netkernel.lang.dpml.ast.IValueVariable;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.nkf.impl.NKFContextImpl;
import org.netkernel.layer0.nkf.impl.NKFRequestReadOnlyImpl;
import org.netkernel.layer0.nkf.impl.NKFResponseReadOnlyImpl;
import org.netkernel.layer0.nkf.impl.ParametersImpl;
import org.netkernel.layer0.urii.ValueSpace;
import org.netkernel.request.IRequest;
import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.request.IResponse;
import org.netkernel.request.IResponseMeta;
import org.netkernel.request.impl.ExpiryFactory;
import org.netkernel.request.impl.MetaImpl;
import org.netkernel.request.impl.RequestResponseFieldsImpl;
import org.netkernel.request.impl.RequestScopeLevelImpl;
import org.netkernel.request.impl.ResponseImpl;
import org.netkernel.urii.ISpace;

public class DPMLValueSpace extends ValueSpace
{
	private final IRequestScopeLevel mLazyEvalScope;
	
	public DPMLValueSpace(int aSize, IRequestScopeLevel aLazyEvalScope)
	{	super(aSize);
		mLazyEvalScope=aLazyEvalScope;
	}
	
	public void addLazyEvaluation(String aIdentifier, IValueVariable aVariable, RuntimeState aRuntimeState)
	{	LazyEval eval=new LazyEval(aVariable, aRuntimeState);
		put(aIdentifier,eval,null);
	}
	
	protected IResponse customHandler(IRequest aRequest, Object aRepresentation, IResponseMeta aMeta )
	{	if (aRepresentation instanceof LazyEval)
		{	LazyEval eval=(LazyEval)aRepresentation;
			NKFRequestReadOnlyImpl req=new NKFRequestReadOnlyImpl(aRequest);
			NKFContextImpl ctx=new NKFContextImpl(getKernel(), this, req, getEndpoint(), ParametersImpl.EMPTY);
			
			//evaluate request in same scope as if it was eagerly evaluated
			IRequestScopeLevel evalScope=mLazyEvalScope;
			//but.... add a possible durable scope injected by endpoint
			IRequestScopeLevel s=aRequest.getRequestScope();
			ISpace pbvSpace=s.getSpace();
			ISpace found=null;
			while (s!=null)
			{	if (s.isDurable() && s.getSpace()!=pbvSpace)
				{	found=s.getSpace();
					break;
				}
				s=s.getParent();
			}
			//eliminate if already exists in scope
			if (found!=null)
			{	s=evalScope;
				while (s!=null)
				{	if (s.getSpace()==found)
					{	found=null;
						break;
					}
					s=s.getParent();
				}
			}
			if (found!=null)
			{	evalScope=RequestScopeLevelImpl.createDurableScopeLevel(found, evalScope);
			}

			ctx.setRequestScope(evalScope);
			ctx.setCWU(eval.mRuntimeState.getScriptIdentifier());
			try
			{	IValue value=eval.mVariable.getValue(ctx, eval.mRuntimeState, aRequest.getRepresentationClass());
				RequestResponseFieldsImpl headers=new RequestResponseFieldsImpl(value.getMeta().getUserMetaData());
				headers.add(IResponseMeta.KEY_NO_CACHE, Boolean.TRUE);
				IResponseMeta meta=new MetaImpl(value.getMeta().getExpiry(),headers);
				return new ResponseImpl(aRequest, value.getRepresentation(),meta);
			}
			catch (Exception e)
			{	return new ResponseImpl(aRequest,e,new MetaImpl(ExpiryFactory.getDependentExpiry()));
			}
		}
		else
		{	return super.customHandler(aRequest, aRepresentation, aMeta);
		}
	}
	
	public boolean equals(Object aOther)
	{	boolean result=super.equals(aOther);
		return result;
	}	
	
	private static class LazyEval
	{
		private final IValueVariable mVariable;
		private final RuntimeState mRuntimeState;
		
		public LazyEval(IValueVariable aVariable, RuntimeState aRuntimeState)
		{	mVariable=aVariable;
			mRuntimeState=aRuntimeState;
		}
	}
}
