package org.netkernel.lang.dpml.endpoint;

import org.netkernel.layer0.meta.impl.IdentifierArgumentMetaImpl;
import org.netkernel.layer0.meta.impl.SourcedArgumentMetaImpl;
import org.netkernel.layer0.nkf.INKFLocale;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFRequestReadOnly;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;
import java.util.*;

public class LogAccessor extends StandardAccessorImpl
{
	public LogAccessor()
	{	declareThreadSafe();
		declareUnhandledExceptionsExpired(false);

		declareArgument(new SourcedArgumentMetaImpl("message", "Message to log", null, new Class[]{String.class}));
		declareArgument(new IdentifierArgumentMetaImpl("level", "Log level INFO|WARNING|ERROR", "INFO"));
		declareArgument(new SourcedArgumentMetaImpl("param", "one or more substitutions into message", null, null));
		declareSourceRepresentation(java.lang.Exception.class);
		declareSourceRepresentation(String.class);
	}
	
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		String message=aContext.source("arg:message",Object.class).toString();
		INKFRequestReadOnly req=aContext.getThisRequest();
		int level = INKFLocale.LEVEL_INFO;
		if (req.argumentExists("level"))
		{	String levelString=req.getArgumentValue("level");
			if (levelString.endsWith("SEVERE"))
			{	level= INKFLocale.LEVEL_SEVERE;
			}
			else if (levelString.endsWith("WARNING"))
			{	level= INKFLocale.LEVEL_WARNING;
			}
		}
		if (req.argumentExists("param"))
		{	List<Object> params = new ArrayList();
			for (int i=0; i<req.getArgumentCount(); i++)
			{	String argName=req.getArgumentName(i);
				if (argName.equals("param"))
				{	String argValue=req.getArgumentValue(i);
					Object param=aContext.source(argValue);
					params.add(param);
				}
			}
			Object[] paramArray = params.toArray();
			message=aContext.formatRaw(message, paramArray);
		}
		aContext.logRaw(level, message);
		INKFResponse response = aContext.createResponseFrom(message);
		response.setNoCache();
	}
}
